""" A module that manages the command line options to the form compiler executable """

from pytools import memoize

@memoize
def get_form_compiler_arguments():
    # define an argument parser.
    from argparse import ArgumentParser
    parser = ArgumentParser(description="Compile UFL files to PDELab C++ code", epilog="Please report bugs to dominic.kempf@iwr.uni-heidelberg.de")
    parser.add_argument("uflfile", type=str, nargs=1, help="the UFL file to compile")
    parser.add_argument("--driver-file", type=str, help="The filename for the generated driver header")
    parser.add_argument("--operator-file", type=str, help="The filename for the generated local operator header")
    parser.add_argument('--version', action='version', version='%(prog)s 0.1')
    parser.add_argument("--numerical-jacobian", action="store_true", help="use numerical jacobians (only makes sense, if uflpdelab for some reason fails to generate analytic jacobians)")

    # These are the options that I deemed necessary in uflpdelab
#     parser.add_argument("--param-class-file", type=str, help="The filename for the generated parameter class header")
#     parser.add_argument("--export-trafo-graphs", action="store_true", help="export expression graphs after the application of each single transformation")
#     parser.add_argument("--parameter-tree", action="store_true", help="have the generated local operate take a Dune::ParameterTree as constructor argument")
#     parser.add_argument("--inifile", type=str, help="The inifile that this program's driver should be hardcoded to. Omit to use a commandline argument ini file")

    # Modify the positional argument to not be a list
    args = vars(parser.parse_args())

    # Delistify the uflfile parameter
    args["uflfile"] = args["uflfile"][0]

    # Turn any relative paths into absolute ones for consistency
    import os.path
    if args["driver_file"]:
        args["driver_file"] = os.path.abspath(args["driver_file"])
    if args["operator_file"]:
        args["operator_file"] = os.path.abspath(args["operator_file"])

    # Return the argument dict. This result is memoized to turn all get_option calls into simple dict lookups.
    return args

def get_option(key, default=None):
    try:
        __IPYTHON__
        return default
    except NameError:
        return get_form_compiler_arguments().get(key, default)